/*
 * PS3 Media Server, for streaming any medias to your PS3.
 * Copyright (C) 2008  A.Brochard
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; version 2
 * of the License only.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package net.pms.newgui;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.debug.FormDebugPanel;
import com.jgoodies.forms.debug.FormDebugUtils;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.sun.jna.Platform;
import net.pms.Messages;
import net.pms.PMS;
import net.pms.configuration.PmsConfiguration;
import net.pms.dlna.DLNAMediaDatabase;
import net.pms.util.FormLayoutUtil;
import net.pms.util.KeyedComboBoxModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.Locale;

public class NavigationShareTab {
	private static final Logger LOGGER = LoggerFactory.getLogger(NavigationShareTab.class);
	public static final String ALL_DRIVES = Messages.getString("FoldTab.0");

    private static final String bottomPanelColSpec = "left:pref, left:120dlu, left:120dlu";
    private static final String bottomPanelRowSpec = "pref,2dlu,pref";

     private JList recursiveFoldersList;
     private JList nonRecursiveFoldersList;
	private DefaultListModel df;
	private JCheckBox hideVideoSettings;
	private JCheckBox hideTranscode;
	private JCheckBox hideLibrary;
	private JCheckBox hideExtensions;
	private JCheckBox hideIrrelevant;
	private JCheckBox hideEngines;
	private JButton but5;
	private JTextField seekPos;
	private JCheckBox genThumb;
	private JCheckBox mplayerThumb;
	private JCheckBox isoThumb;
	private JCheckBox imgThumb;
	private JCheckBox cacheOn;
	private JCheckBox browseArchive;
	private JComboBox sortMethod;
	private JComboBox audioThumb;
	private JTextField defaultThumbFolder;
	private JCheckBox iphotoShow;
	private JCheckBox apertureShow;
	private JCheckBox itunesShow;
	private JButton browseFileSystem;
	private JButton cacheReset;
    private JPanel panel;

	public DefaultListModel getDf() {
		return df;
	}
	private final PmsConfiguration configuration;

	NavigationShareTab(PmsConfiguration configuration) {
		this.configuration = configuration;
	}

	private void updateModel() {
		if (df.size() == 1 && df.getElementAt(0).equals(ALL_DRIVES)) {
			configuration.setFolders("");
		} else {
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < df.size(); i++) {
				if (i > 0) {
					sb.append(",");
				}
				String entry = (String) df.getElementAt(i);
				// escape embedded commas. note: backslashing isn't safe as it conflicts with
				// Windows path separators:
				// http://ps3mediaserver.org/forum/viewtopic.php?f=14&t=8883&start=250#p43520
				sb.append(entry.replace(",", "&comma;"));
			}
			configuration.setFolders(sb.toString());
		}
	}

    private static final String topPanelColSpec = "80dlu,4dlu,6dlu,4dlu,30dlu,30dlu,10dlu,10dlu,4dlu,80dlu,4dlu,6dlu,default";
    private static final String topPanelRowSpec = "pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu,pref,2dlu";

    public JComponent build() {
        // Apply the orientation for the locale
        Locale locale = new Locale(configuration.getLanguage());
        ComponentOrientation orientation = ComponentOrientation.getOrientation(locale);
        String colSpec = FormLayoutUtil.getColSpec(topPanelColSpec, orientation);

        // Attempt new top panel layout and build
        FormLayout topPanel = new FormLayout(topPanelColSpec, topPanelRowSpec);
        PanelBuilder topPanelBuilder = new PanelBuilder(topPanel);
        topPanelBuilder.setBorder(Borders.DLU4_BORDER);
        topPanelBuilder.setOpaque(true);
        topPanel.setRowGroups(new int[][]{{3,5}});
        topPanel.addGroupedRow(7);
        topPanel.addGroupedRow(9);
        topPanel.addGroupedRow(13);
        topPanel.addGroupedRow(15);
        topPanel.addGroupedRow(17);
        topPanel.addGroupedRow(19);
        topPanel.addGroupedRow(21);
        topPanel.addGroupedRow(23);
        topPanel.addGroupedRow(27);
        CellConstraints cc = new CellConstraints();
        initSimpleComponents(cc);
        topPanelBuilder = initSharedFoldersGuiComponents(cc);

        // Attempt new top panel GUI build
        JComponent thumbSep = topPanelBuilder.addSeparator(Messages.getString("FoldTab.13"), cc.xyw(1,1,12));
        thumbSep = (JComponent) thumbSep.getComponent(0);
        thumbSep.setFont(thumbSep.getFont().deriveFont(Font.BOLD));
        // First row
        topPanelBuilder.addLabel(Messages.getString("NetworkTab.2"), cc.xyw(1, 3, 1));
        topPanelBuilder.add(genThumb, cc.xyw(3, 3, 1));
        topPanelBuilder.addLabel(Messages.getString("NetworkTab.16"), cc.xyw(5, 3, 3));
        topPanelBuilder.add(seekPos, cc.xyw(8, 3, 1));
        topPanelBuilder.addLabel(Messages.getString("NetworkTab.56"), cc.xyw(10, 3, 3));
        // Second Row
        topPanelBuilder.addLabel(Messages.getString("FoldTab.14"), cc.xyw(1, 5, 1));
        topPanelBuilder.add(mplayerThumb, cc.xyw(3, 5, 1));
        topPanelBuilder.addLabel(Messages.getString("FoldTab.27"), cc.xyw(5, 5, 2));
        topPanelBuilder.add(audioThumb, cc.xyw(8, 5, 2));
        topPanelBuilder.addLabel("", cc.xyw(10, 5, 3));
/**
 * // Third Row
        topPanelBuilder.add(image_thumb, FormLayoutUtil.flip(cc.xyw(1, 7, 3), colSpec, orientation));
        topPanelBuilder.addLabel(Messages.getString("FoldTab.26"), FormLayoutUtil.flip(cc.xyw(5, 7, 2), colSpec, orientation));
        topPanelBuilder.add(defaultThumbFolder, FormLayoutUtil.flip(cc.xyw(7, 7, 1), colSpec, orientation));
        topPanelBuilder.add(select, FormLayoutUtil.flip(cc.xyw(8, 7, 1), colSpec, orientation));
        // Fourth Row
        topPanelBuilder.add(dvdiso_thumb, FormLayoutUtil.flip(cc.xyw(1, 9, 3), colSpec, orientation));

        JComponent navSep = topPanelBuilder.addSeparator(Messages.getString("NetworkTab.15"), FormLayoutUtil.flip(cc.xyw(1, 13, 12), colSpec, orientation));
        navSep = (JComponent) navSep.getComponent(0);
        navSep.setFont(navSep.getFont().deriveFont(Font.BOLD));

        topPanelBuilder.add(archive, FormLayoutUtil.flip(cc.xyw(1, 15, 3), colSpec, orientation));
        topPanelBuilder.addLabel(Messages.getString("FoldTab.18"), FormLayoutUtil.flip(cc.xyw(5, 15, 1), colSpec, orientation));
        topPanelBuilder.add(sortmethod, FormLayoutUtil.flip(cc.xyw(6, 15, 3), colSpec, orientation));
        topPanelBuilder.add(hidevideosettings, FormLayoutUtil.flip(cc.xyw(10, 15, 3), colSpec, orientation));

        topPanelBuilder.add(hideextensions, FormLayoutUtil.flip(cc.xyw(1, 17, 3), colSpec, orientation));
        topPanelBuilder.add(hidetranscode, FormLayoutUtil.flip(cc.xyw(10, 17, 3), colSpec, orientation));

        topPanelBuilder.add(itunes, FormLayoutUtil.flip(cc.xyw(1, 19, 3), colSpec, orientation));
        topPanelBuilder.add(hideengines, FormLayoutUtil.flip(cc.xyw(10, 19, 3), colSpec, orientation));

        topPanelBuilder.add(iphoto, FormLayoutUtil.flip(cc.xyw(1, 21, 3), colSpec, orientation));
        topPanelBuilder.add(hideemptyfolders, FormLayoutUtil.flip(cc.xyw(10, 21, 3), colSpec, orientation));

        topPanelBuilder.add(aperture, FormLayoutUtil.flip(cc.xyw(1, 23, 3), colSpec, orientation));

        topPanelBuilder.add(cacheenable, FormLayoutUtil.flip(cc.xyw(1, 27, 3), colSpec, orientation));
        topPanelBuilder.add(cachereset, FormLayoutUtil.flip(cc.xyw(7, 27, 2), colSpec, orientation));
        topPanelBuilder.add(hidemedialibraryfolder, FormLayoutUtil.flip(cc.xyw(10, 27, 3), colSpec, orientation));

//        topPanelBuilder.add(recursiveFoldersPanelBuilder.getPanel(), FormLayoutUtil.flip(cc.xyw(1, 29, 12), colSpec, orientation));
//        topPanelBuilder.add(nonRecursiveFoldersPanelBuilder.getPanel(), FormLayoutUtil.flip(cc.xyw(1,31,12), colSpec, orientation));
**/
        //JPanel panel = topPanelBuilder.getPanel();
        JPanel panel = new FormDebugPanel();

        // Apply the orientation to the panel and all components in it
        panel.applyComponentOrientation(orientation);

        JScrollPane scrollPane = new JScrollPane(panel,JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,	JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scrollPane.setBorder(BorderFactory.createEmptyBorder());
        FormDebugUtils.dumpAll(panel);
        return scrollPane;
    }

	private void initSimpleComponents(CellConstraints cc) {
        /* THUMBNAILS SECTION */
        // Generate checkboxes
        genThumb = new JCheckBox();
        genThumb.setContentAreaFilled(false);
        genThumb.addItemListener(new ItemListener() {
			@Override public void itemStateChanged(ItemEvent e) {
				configuration.setThumbnailGenerationEnabled((e.getStateChange() == ItemEvent.SELECTED));
			}
		});
        if (configuration.isThumbnailGenerationEnabled()) {
            genThumb.setSelected(true);
        }
        mplayerThumb = new JCheckBox();
        mplayerThumb.setContentAreaFilled(false);
        mplayerThumb.addItemListener(new ItemListener() {
            @Override public void itemStateChanged(ItemEvent e) {
                configuration.setUseMplayerForVideoThumbs((e.getStateChange() == ItemEvent.SELECTED));
            }
		});
        if (configuration.isUseMplayerForVideoThumbs()) {
            mplayerThumb.setSelected(true);
        }
		isoThumb = new JCheckBox();
		isoThumb.setContentAreaFilled(false);
		isoThumb.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setDvdIsoThumbnails((e.getStateChange() == ItemEvent.SELECTED));
			}
		});
		if (configuration.isDvdIsoThumbnails()) {
			isoThumb.setSelected(true);
		}

        //ThumbnailSeekPos
        seekPos = new JTextField("" + configuration.getThumbnailSeekPos());
        seekPos.addKeyListener(new KeyListener() {
            @Override public void keyPressed(KeyEvent e) {}
            @Override public void keyTyped(KeyEvent e) {}
            @Override public void keyReleased(KeyEvent e) {
                try {
                    int ab = Integer.parseInt(seekPos.getText());
                    configuration.setThumbnailSeekPos(ab);
                } catch (NumberFormatException nfe) {
                    LOGGER.debug("Could not parse thumbnail seek position from \"" + seekPos.getText() + "\"");
			}
            }
		});
        // DvdIsoThumbnails

		// ImageThumbnailsEnabled
		imgThumb = new JCheckBox(Messages.getString("FoldTab.21"));
		imgThumb.setContentAreaFilled(false);
		imgThumb.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setImageThumbnailsEnabled((e.getStateChange() == ItemEvent.SELECTED));
			}
		});
		if (configuration.getImageThumbnailsEnabled()) {
			imgThumb.setSelected(true);
		}

		// AudioThumbnailMethod
		final KeyedComboBoxModel thumbKCBM = new KeyedComboBoxModel(new Object[]{"0", "1", "2"}, new Object[]{Messages.getString("FoldTab.35"), Messages.getString("FoldTab.23"), Messages.getString("FoldTab.24")});
		audioThumb = new JComboBox(thumbKCBM);
		audioThumb.setEditable(false);

		thumbKCBM.setSelectedKey("" + configuration.getAudioThumbnailMethod());

		audioThumb.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {

					try {
						configuration.setAudioThumbnailMethod(Integer.parseInt((String) thumbKCBM.getSelectedKey()));
					} catch (NumberFormatException nfe) {
						LOGGER.debug("Could not parse audio thumbnail method from \"" + thumbKCBM.getSelectedKey() + "\"");
					}

				}
			}
		});

		// AlternateCoverArtFolder
		defaultThumbFolder = new JTextField(configuration.getAlternateThumbFolder());
		defaultThumbFolder.addKeyListener(new KeyListener() {
			@Override
			public void keyPressed(KeyEvent e) {
			}

			@Override
			public void keyTyped(KeyEvent e) {
			}

			@Override
			public void keyReleased(KeyEvent e) {
				configuration.setAlternateThumbFolder(defaultThumbFolder.getText());
			}
		});


		// AlternateCoverArtFolder: select
		browseFileSystem = new JButton("...");
		browseFileSystem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				JFileChooser chooser;
				try {
					chooser = new JFileChooser();
				} catch (Exception ee) {
					chooser = new JFileChooser(new RestrictedFileSystemView());
				}
				chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int returnVal = chooser.showDialog((Component) e.getSource(), Messages.getString("FoldTab.28"));
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					defaultThumbFolder.setText(chooser.getSelectedFile().getAbsolutePath());
					configuration.setAlternateThumbFolder(chooser.getSelectedFile().getAbsolutePath());
				}
			}
		});
                /* NAVIGATION/PARSING SECTION */
		// HideVideoSettings
		hideVideoSettings = new JCheckBox(Messages.getString("FoldTab.6"));
		hideVideoSettings.setContentAreaFilled(false);
		if (configuration.getHideVideoSettings()) {
			hideVideoSettings.setSelected(true);
		}
		hideVideoSettings.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setHideVideoSettings((e.getStateChange() == ItemEvent.SELECTED));
			}
		});

		hideTranscode = new JCheckBox(Messages.getString("FoldTab.33"));
		hideTranscode.setContentAreaFilled(false);
		if (configuration.getHideTranscodeEnabled()) {
			hideTranscode.setSelected(true);
		}
		hideTranscode.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setHideTranscodeEnabled((e.getStateChange() == ItemEvent.SELECTED));
			}
		});

		hideLibrary = new JCheckBox(Messages.getString("FoldTab.32"));
		hideLibrary.setContentAreaFilled(false);
		if (configuration.isHideMediaLibraryFolder()) {
			hideLibrary.setSelected(true);
		}
		hideLibrary.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setHideMediaLibraryFolder((e.getStateChange() == ItemEvent.SELECTED));
			}
		});

		browseArchive = new JCheckBox(Messages.getString("NetworkTab.1"));
		browseArchive.setContentAreaFilled(false);
		browseArchive.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setArchiveBrowsing(e.getStateChange() == ItemEvent.SELECTED);
			}
		});
		if (configuration.isArchiveBrowsing()) {
			browseArchive.setSelected(true);
		}

		cacheReset = new JButton(Messages.getString("NetworkTab.18"));

		cacheOn = new JCheckBox(Messages.getString("NetworkTab.17"));
		cacheOn.setContentAreaFilled(false);
		cacheOn.setSelected(configuration.getUseCache());
		cacheOn.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setUseCache((e.getStateChange() == ItemEvent.SELECTED));
				cacheReset.setEnabled(configuration.getUseCache());
				if ((LooksFrame) PMS.get().getFrame() != null) {
					((LooksFrame) PMS.get().getFrame()).getFt().setScanLibraryEnabled(configuration.getUseCache());
				}
			}
		});

		cacheReset.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				int option = JOptionPane.showConfirmDialog(
					(Component) PMS.get().getFrame(),
					Messages.getString("NetworkTab.13") + Messages.getString("NetworkTab.19"),
					"Question",
					JOptionPane.YES_NO_OPTION);
				if (option == JOptionPane.YES_OPTION) {
					PMS.get().getDatabase().init(true);
				}

			}
		});
		cacheReset.setEnabled(configuration.getUseCache());

		// HideExtensions
		hideExtensions = new JCheckBox(Messages.getString("FoldTab.5"));
		hideExtensions.setContentAreaFilled(false);
		if (configuration.isHideExtensions()) {
			hideExtensions.setSelected(true);
		}
		hideExtensions.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setHideExtensions((e.getStateChange() == ItemEvent.SELECTED));
			}
		});

		// HideEngineNames
		hideEngines = new JCheckBox(Messages.getString("FoldTab.8"));
		hideEngines.setContentAreaFilled(false);
		if (configuration.isHideEngineNames()) {
			hideEngines.setSelected(true);
		}
		hideEngines.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setHideEngineNames((e.getStateChange() == ItemEvent.SELECTED));
			}
		});

		// HideEmptyFolders
		hideIrrelevant = new JCheckBox(Messages.getString("FoldTab.31"));
		hideIrrelevant.setContentAreaFilled(false);
		if (configuration.isHideEmptyFolders()) {
			hideIrrelevant.setSelected(true);
		}
		hideIrrelevant.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setHideEmptyFolders((e.getStateChange() == ItemEvent.SELECTED));
			}
		});

		// ItunesEnabled
		itunesShow = new JCheckBox(Messages.getString("FoldTab.30"));
		itunesShow.setContentAreaFilled(false);
		if (configuration.getItunesEnabled()) {
			itunesShow.setSelected(true);
		}
		if (!(Platform.isMac() || Platform.isWindows())) {
			itunesShow.setEnabled(false);
		}
		itunesShow.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setItunesEnabled((e.getStateChange() == ItemEvent.SELECTED));
			}
		});

		// IphotoEnabled
		iphotoShow = new JCheckBox(Messages.getString("FoldTab.29"));
		iphotoShow.setContentAreaFilled(false);
		if (configuration.getIphotoEnabled()) {
			iphotoShow.setSelected(true);
		}
		if (!Platform.isMac()) {
			iphotoShow.setEnabled(false);
		}
		iphotoShow.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setIphotoEnabled((e.getStateChange() == ItemEvent.SELECTED));
			}
		});

		// ApertureEnabled
		apertureShow = new JCheckBox(Messages.getString("FoldTab.34"));
		apertureShow.setContentAreaFilled(false);
		if (configuration.getApertureEnabled()) {
			apertureShow.setSelected(true);
		}
		if (!Platform.isMac()) {
			apertureShow.setEnabled(false);
		}
		apertureShow.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				configuration.setApertureEnabled((e.getStateChange() == ItemEvent.SELECTED));
			}
		});

		// sort method
		final KeyedComboBoxModel kcbm = new KeyedComboBoxModel(
			new Object[]{
				"0", // alphabetical
				"4", // natural sort
				"3", // ASCIIbetical
				"1", // newest first
				"2"  // oldest first
			},
			new Object[]{
				Messages.getString("FoldTab.15"),
				Messages.getString("FoldTab.22"),
				Messages.getString("FoldTab.20"),
				Messages.getString("FoldTab.16"),
				Messages.getString("FoldTab.17")
			}
		);
		sortMethod = new JComboBox(kcbm);
		sortMethod.setEditable(false);
		kcbm.setSelectedKey("" + configuration.getSortMethod());

		sortMethod.addItemListener(new ItemListener() {
			@Override
			public void itemStateChanged(ItemEvent e) {
				if (e.getStateChange() == ItemEvent.SELECTED) {

					try {
						configuration.setSortMethod(Integer.parseInt((String) kcbm.getSelectedKey()));
					} catch (NumberFormatException nfe) {
						LOGGER.debug("Could not parse sort method from \"" + kcbm.getSelectedKey() + "\"");
					}

				}
			}
		});
	}

	private static final String sharedFoldersColSpec = "left:pref, left:pref, pref, pref, pref, 0:grow";
	private static final String sharedFoldersRowSpec = "p, 1dlu, p, 1dlu, fill:default:grow";

    private PanelBuilder initSharedFoldersGuiComponents(CellConstraints cc) {
		// Apply the orientation for the locale
		Locale locale = new Locale(configuration.getLanguage());
		ComponentOrientation orientation = ComponentOrientation.getOrientation(locale);

          // Layout and build the Recursives folders list area
		String colSpec = FormLayoutUtil.getColSpec(sharedFoldersColSpec, orientation);
		FormLayout foldersLayout = new FormLayout(colSpec, sharedFoldersRowSpec);
		PanelBuilder recursiveFoldersPanelBuilder = new PanelBuilder(foldersLayout);
		recursiveFoldersPanelBuilder.setOpaque(true);

		JComponent cmp = recursiveFoldersPanelBuilder.addSeparator(Messages.getString("FoldTab.7"), FormLayoutUtil.flip(cc.xyw(1, 1, 6), colSpec, orientation));
		cmp = (JComponent) cmp.getComponent(0);
		cmp.setFont(cmp.getFont().deriveFont(Font.BOLD));

		JButton but = new JButton(LooksFrame.readImageIcon("button-adddirectory.png"));
		but.setToolTipText(Messages.getString("FoldTab.9"));
		but.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				JFileChooser chooser;
				try {
					chooser = new JFileChooser();
				} catch (Exception ee) {
					chooser = new JFileChooser(new RestrictedFileSystemView());
				}
				chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int returnVal = chooser.showOpenDialog((Component) e.getSource());
				if (returnVal == JFileChooser.APPROVE_OPTION) {

                   ((DefaultListModel) recursiveFoldersList.getModel()).add(recursiveFoldersList.getModel().getSize(), chooser.getSelectedFile().getAbsolutePath());
					if (recursiveFoldersList.getModel().getElementAt(0).equals(ALL_DRIVES)) {
						((DefaultListModel) recursiveFoldersList.getModel()).remove(0);
					}
					updateModel();
				}
			}
		});
		recursiveFoldersPanelBuilder.add(but, FormLayoutUtil.flip(cc.xy(1, 3), colSpec, orientation));

		JButton but2 = new JButton(LooksFrame.readImageIcon("button-remove.png"));
		but2.setToolTipText(Messages.getString("FoldTab.36"));
		//but2.setBorder(BorderFactory.createEtchedBorder());
		but2.addActionListener(new java.awt.event.ActionListener() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (recursiveFoldersList.getSelectedIndex() > -1) {
					((DefaultListModel) recursiveFoldersList.getModel()).remove(recursiveFoldersList.getSelectedIndex());
					if (recursiveFoldersList.getModel().getSize() == 0) {
						((DefaultListModel) recursiveFoldersList.getModel()).add(0, ALL_DRIVES);
					}
					updateModel();
				}
			}
		});
		recursiveFoldersPanelBuilder.add(but2, FormLayoutUtil.flip(cc.xy(2, 3), colSpec, orientation));

		JButton but3 = new JButton(LooksFrame.readImageIcon("button-arrow-down.png"));
		but3.setToolTipText(Messages.getString("FoldTab.12"));
		// but3.setBorder(BorderFactory.createEmptyBorder());
		but3.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				DefaultListModel model = ((DefaultListModel) recursiveFoldersList.getModel());
				for (int i = 0; i < model.size() - 1; i++) {
					if (recursiveFoldersList.isSelectedIndex(i)) {
						String value = model.get(i).toString();
						model.set(i, model.get(i + 1));
						model.set(i + 1, value);
						recursiveFoldersList.setSelectedIndex(i + 1);
						updateModel();
						break;
					}
				}
			}
		});

        recursiveFoldersPanelBuilder.add(but3, FormLayoutUtil.flip(cc.xy(3, 3), colSpec, orientation));
        JButton but4 = new JButton(LooksFrame.readImageIcon("button-arrow-up.png"));
        but4.setToolTipText(Messages.getString("FoldTab.12"));
        //  but4.setBorder(BorderFactory.createEmptyBorder());
        but4.addActionListener(new ActionListener() {
            @Override
                public void actionPerformed(ActionEvent e) {
				DefaultListModel model = ((DefaultListModel) recursiveFoldersList.getModel());
				for (int i = 1; i < model.size(); i++) {
					if (recursiveFoldersList.isSelectedIndex(i)) {
						String value = model.get(i).toString();

						model.set(i, model.get(i - 1));
						model.set(i - 1, value);
						recursiveFoldersList.setSelectedIndex(i - 1);
						updateModel();
						break;

					}
				}
			}
		});
		recursiveFoldersPanelBuilder.add(but4, FormLayoutUtil.flip(cc.xy(4, 3), colSpec, orientation));

		but5 = new JButton(LooksFrame.readImageIcon("button-scan.png"));
		but5.setToolTipText(Messages.getString("FoldTab.2"));
		//but5.setBorder(BorderFactory.createEmptyBorder());
		but5.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (configuration.getUseCache()) {
					DLNAMediaDatabase database = PMS.get().getDatabase();

					if (database != null) {
						if (!database.isScanLibraryRunning()) {
							int option = JOptionPane.showConfirmDialog(
								(Component) PMS.get().getFrame(),
								Messages.getString("FoldTab.3") + Messages.getString("FoldTab.4"),
								"Question",
								JOptionPane.YES_NO_OPTION);
							if (option == JOptionPane.YES_OPTION) {
								database.scanLibrary();
								but5.setIcon(LooksFrame.readImageIcon("button-scan-busy.gif"));
							}
						} else {
							int option = JOptionPane.showConfirmDialog(
								(Component) PMS.get().getFrame(),
								Messages.getString("FoldTab.10"),
								"Question",
								JOptionPane.YES_NO_OPTION);
							if (option == JOptionPane.YES_OPTION) {
								database.stopScanLibrary();
								PMS.get().getFrame().setStatusLine(null);
								but5.setIcon(LooksFrame.readImageIcon("button-scan.png"));
							}
						}
					}
				}
			}
		});

		recursiveFoldersPanelBuilder.add(but5, FormLayoutUtil.flip(cc.xy(5, 3), colSpec, orientation));

                                but5.setEnabled(configuration.getUseCache());
		df = new DefaultListModel();
		File[] folders = PMS.get().getFoldersConf(false);
		if (folders != null && folders.length > 0) {
			for (File file : folders) {
				df.addElement(file.getAbsolutePath());
			}
		} else {
			df.addElement(ALL_DRIVES);
		}

        recursiveFoldersList = new JList();
        recursiveFoldersList.setModel(df);
        JScrollPane pane = new JScrollPane(recursiveFoldersList);
        recursiveFoldersPanelBuilder.add(pane, FormLayoutUtil.flip(cc.xyw(1, 5, 6), colSpec, orientation));
        return recursiveFoldersPanelBuilder;
	}

	public void setScanLibraryEnabled(boolean enabled) {
		but5.setEnabled(enabled);
		but5.setIcon(LooksFrame.readImageIcon("button-scan.png"));
	}
}
